;;  ----------------------------------------------------------------  ;;
;;                 Nagoya Institute of Technology and                 ;;
;;                     Carnegie Mellon University                     ;;
;;                         Copyright (c) 2002                         ;;
;;                        All Rights Reserved.                        ;;
;;                                                                    ;;
;;  Permission is hereby granted, free of charge, to use and          ;;
;;  distribute this software and its documentation without            ;;
;;  restriction, including without limitation the rights to use,      ;;
;;  copy, modify, merge, publish, distribute, sublicense, and/or      ;;
;;  sell copies of this work, and to permit persons to whom this      ;;
;;  work is furnished to do so, subject to the following conditions:  ;;
;;                                                                    ;;
;;    1. The code must retain the above copyright notice, this list   ;;
;;       of conditions and the following disclaimer.                  ;;
;;                                                                    ;;
;;    2. Any modifications must be clearly marked as such.            ;;
;;                                                                    ;;
;;    3. Original authors' names are not deleted.                     ;;
;;                                                                    ;;
;;    4. The authors' names are not used to endorse or promote        ;;
;;       products derived from this software without specific prior   ;;
;;       written permission.                                          ;;
;;                                                                    ;;
;;  NAGOYA INSTITUTE OF TECHNOLOGY, CARNEGIE MELLON UNIVERSITY AND    ;;
;;  THE CONTRIBUTORS TO THIS WORK DISCLAIM ALL WARRANTIES WITH        ;;
;;  REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF      ;;
;;  MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL NAGOYA INSTITUTE   ;;
;;  OF TECHNOLOGY, CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS    ;;
;;  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR   ;;
;;  ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        ;;
;;  PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER    ;;
;;  TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR  ;;
;;  PERFORMANCE OF THIS SOFTWARE.                                     ;;
;;                                                                    ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;     A voice based on "HTS" HMM-Based Speech Synthesis System.      ;;
;;          Author :  Alan W Black                                    ;;
;;          Date   :  August 2002                                     ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Try to find the directory where the voice is, this may be from
;;; .../festival/lib/voices/ or from the current directory
(if (assoc 'eki_et_tonu_hts voice-locations)
    (defvar eki_et_tonu_hts::hts_dir 
      (cdr (assoc 'eki_et_tonu_hts voice-locations)))
    (defvar eki_et_tonu_hts::hts_dir (string-append (pwd) "/")))

(defvar eki_et_tonu::clunits_dir eki_et_tonu_hts::hts_dir)
(defvar eki_et_tonu::clunits_loaded nil)
(defvar festvox_debug 0)

;;; Did we succeed in finding it
(if (not (probe_file (path-append eki_et_tonu_hts::hts_dir "festvox/")))
    (begin
     (format stderr "eki_et_tonu_hts::hts: Can't find voice scm files they are not in\n")
     (format stderr "   %s\n" (path-append  eki_et_tonu_hts::hts_dir "festvox/"))
     (format stderr "   Either the voice isn't linked in Festival library\n")
     (format stderr "   or you are starting festival in the wrong directory\n")
     (error)))

;;;  Add the directory contains general voice stuff to load-path
(set! load-path (cons (path-append eki_et_tonu_hts::hts_dir "festvox/") 
		      load-path))

(set! hts_data_dir (path-append eki_et_tonu_hts::hts_dir "hts/"))

(set! hts_feats_list
      (load (path-append hts_data_dir "feat.list") t))

(require 'hts)
(require_module 'hts_engine)


(defvar eki_et_eou_tree 
  '((lisp_max_num_tokens > 200)
    ((1))
    ((n.whitespace matches ".*\n.*\n\\(.\\|\n\\)*");; significant break (2 nls)
     ((1))
     ((name matches "-+")
     ((1))
     ((punc matches ".*[\\?:!;].*")
      ((1))
      ((punc matches ".*\\..*")
       ((punc matches "..+");; longer punctuation string
	((punc matches "\\..*,")  ;; for U.S.S.R., like tokens
	 ((0))
	 ((1)))
	;; This is to distinguish abbreviations vs periods
	;; These are heuristics
	((name matches "\\(.*\\..*\\|[A-Z][A-Za-z]?[A-Za-z]?\\|etc\\)");; an abbreviation
	 ((n.whitespace is " ")
	  ((0));; if abbrev single space isn't enough for break
	  ((n.name matches "[A-Zܦ0-9].*")
	   ((1))
	   ((0))))
	 ((n.whitespace is " ");; if it doesn't look like an abbreviation
	  ((n.name matches "[A-Zܦ0-9].*");; single space and non-cap is no break
	   ((1))
	   ((0)))
	  ((1)))))
       ((0)))))))
  "eou_tree
End of utterance tree.  A decision tree used to determine if the given
token marks the end of an utterance.  It may look one token ahead to
do this. [see Utterance chunking]")

(set! eou_tree eki_et_eou_tree)

;;; Voice specific parameter are defined in each of the following
;;; files

(require 'eki_et_tonu_phoneset)
(require 'eki_et_tonu_tokenizer)
(require 'eki_et_tonu_tokendata)
(require 'eki_et_tonu_tagger)
(require 'eki_et_tonu_lexicon)
(require 'eki_et_tonu_phrasing)
(require 'eki_et_tonu_intonation)
(require 'eki_et_tonu_duration)
(require 'eki_et_tonu_f0model)
(require 'eki_et_tonu_morphdata)
(require 'eki_et_tonu_morph)

;; ... and others as required


(define (eki_et_tonu_hts::voice_reset)
  "(eki_et_tonu_hts::voice_reset)
Reset global variables back to previous voice."
  (eki_et_tonu::reset_phoneset)
  (eki_et_tonu::reset_tokenizer)
  (eki_et_tonu::reset_tagger)
  (eki_et_tonu::reset_lexicon)
  (eki_et_tonu::reset_phrasing)
  (eki_et_tonu::reset_intonation)
  (eki_et_tonu::reset_duration)
  (eki_et_tonu::reset_f0model)
;  (eki_et_tonu::reset_other)

  t
)

(set! eki_et_tonu_hts::hts_feats_list
      (load (path-append hts_data_dir "feat.list") t))

(set! eki_et_tonu_hts::hts_engine_params
      (list
       (list "-md" (path-append hts_data_dir "dur.pdf"))
       (list "-mm" (path-append hts_data_dir "mgc.pdf"))
       (list "-mf" (path-append hts_data_dir "lf0.pdf"))
       (list "-td" (path-append hts_data_dir "tree-dur.inf"))
       (list "-tm" (path-append hts_data_dir "tree-mgc.inf"))
       (list "-tf" (path-append hts_data_dir "tree-lf0.inf"))
       (list "-dm1" (path-append hts_data_dir "mgc.win1"))
       (list "-dm2" (path-append hts_data_dir "mgc.win2"))
       (list "-dm3" (path-append hts_data_dir "mgc.win3"))
       (list "-df1" (path-append hts_data_dir "lf0.win1"))
       (list "-df2" (path-append hts_data_dir "lf0.win2"))
       (list "-df3" (path-append hts_data_dir "lf0.win3"))
       (list "-cm" (path-append hts_data_dir "gv-mgc.pdf"))
       (list "-cf" (path-append hts_data_dir "gv-lf0.pdf"))
       (list "-em" (path-append hts_data_dir "tree-gv-mgc.inf"))
       (list "-ef" (path-append hts_data_dir "tree-gv-lf0.inf"))
       (list "-k" (path-append hts_data_dir "gv-switch.inf"))
       '("-s" 48000.0) ; 45
       '("-p" 220.0) ; 210
       '("-a" 0.55) ; porgand
       '("-g" 0.0)
       '("-b" 0.0)
       '("-u" 0.5)
       
;       '("-s" 32000.0)
;       '("-p" 160.0)
;       '("-a" 0.45)
;       '("-g" 0.0)
;       '("-b" 0.0)
;       '("-u" 0.5)
       
       
       ))
       
       
(define (redef_params vs vp va)
	(set! l '())
	(set! l (append l (list (car hts_engine_params))))
  (mapcar
   	(lambda (s)
		(cond
			((string-equal "-s" (car s))
				(set! l (append l (list (list "-s" vs)))))
			((string-equal "-p" (car s))
				(set! l (append l (list (list "-p" vp)))))
			((string-equal "-a" (car s))
				(set! l (append l (list (list "-a" va)))))
			(t (set! l (append l (list s))))))
			(cdr hts_engine_params))
		(set! hts_engine_params l))
       

;; This function is called to setup a voice.  It will typically
;; simply call functions that are defined in other files in this directory
;; Sometime these simply set up standard Festival modules othertimes
;; these will be specific to this voice.
;; Feel free to add to this list if your language requires it

(define (voice_eki_et_tonu_hts)
  "(voice_eki_et_tonu_hts)
Define voice for limited domain: us."
  ;; *always* required
  (voice_reset)

  ;; Select appropriate phone set
  (eki_et_tonu::select_phoneset)

  ;; Select appropriate tokenization
  (eki_et_tonu::select_tokenizer)

  ;; For part of speech tagging
  (eki_et_tonu::select_tagger)

  (eki_et_tonu::select_lexicon)
  ;; For hts selection you probably don't want vowel reduction
  ;; the unit selection will do that
  (if (string-equal "americanenglish" (Param.get 'Language))
      (set! postlex_vowel_reduce_cart_tree nil))

  (eki_et_tonu::select_phrasing)

  (eki_et_tonu::select_intonation)

  (eki_et_tonu::select_duration)

  (eki_et_tonu::select_f0model)

  ;; Waveform synthesis model: hts
  (set! hts_engine_params eki_et_tonu_hts::hts_engine_params)
  (set! hts_feats_list eki_et_tonu_hts::hts_feats_list)
  (Parameter.set 'Synth_Method 'HTS)

  ;; This is where you can modify power (and sampling rate) if desired
;  (set! after_synth_hooks nil)
  (set! after_synth_hooks
      (list
        (lambda (utt)
          (utt.wave.resample utt 48000))))

  (set! current_voice_reset eki_et_tonu_hts::voice_reset)

  (set! current-voice 'eki_et_tonu_hts)
)

(provide 'eki_et_tonu_hts)

