# package Lingua::ET::Lyhvrk;
package Lyhvrk;
use strict;
use warnings;
use utf8;
require Exporter;

our @ISA = qw(Exporter);

our @EXPORT_OK = qw(
	lyhend
	sonadeks
);

our %EXPORT_TAGS = ( 'all' => \@EXPORT_OK );

our @EXPORT = qw(lyhend sonadeks);

our $VERSION = '0.01';

my %tahed_eesti = (
"a" => 'aa',
"b" => 'bee',
"c" => 'tsee',
"d" => 'dee',
"e" => 'ee',
"f" => 'ef',
"g" => 'gee',
"h" => 'haa',
"i" => 'ii',
"j" => 'jott',
"k" => 'kaa',
"l" => 'ell',
"m" => 'emm',
"n" => 'enn',
"o" => 'oo',
"p" => 'pee',
"q" => 'kuu',
"r" => 'err',
"s" => 'ess',
"š" => 'šaa',
"z" => 'tsett',
"ž" => 'žee',
"t" => 'tee',
"u" => 'uu',
"v" => 'vee',
"w" => 'kaksisvee',
"õ" => 'õõ',
"ä" => 'ää',
"ö" => 'öö',
"ü" => 'üü',
"x" => 'iks',
"y" => 'igrek',
);

my %tahed_inglise = (
"a" => 'ei',
"b" => 'bii',
"c" => 'sii',
"d" => 'dii',
"e" => 'ii',
"f" => 'ef',
"g" => 'džii',
"h" => 'eitš',
"i" => 'ai',
"j" => 'džei',
"k" => 'kei',
"l" => 'ell',
"m" => 'emm',
"n" => 'enn',
"o" => 'õu',
"p" => 'pii',
"q" => 'kjuu',
"r" => 'aar',
"s" => 'ess',
"t" => 'tii',
"u" => 'juu',
"v" => 'vii',
"w" => 'dablju',
"x" => 'eks',
"y" => 'vai',
"z" => 'tsett',
);

# Mõelda võiks veel, kuidas TPI -> tee pee ii, aga TPIs -> [käis] tipis või
# STAS -> ess tee aa ess, aga STASi -> stasi [nuhk]

# eesti reeglitega tähthaaval hääldatavad akronüümid
# lubatud on ka väiketäheline käändelõpp kriipsuga või ilma (TVs, ERR-ile),
# samuti liitsõna esiotsana kriipsuga (DVD-plaadid)
my @akro_eesti = qw (
AA AAA ADL ADSL AK AP
BSA
CCTV CIA CMV CO CSKA CV
DDT DNA DV DVD
EEC EEG EKG EL ELF ELO EM EMO ENSV EOK EON ERR ETV EV EVP EVS
FDA FSB
GM GMO GMT GPS GPU GRU
HBO HEV HIV HTML
ICES ID IMF iOS IQ IRA IRL ISBN IT ITV
KGB KKK
LCD LKA LNG LSD
MM MRI MSN MTV MTÜ
NB NBA NHL NKVD NMKÜ NL NLKP NOAA NSA NSV NSVL NYC 
ODC OECD ORF OSCE OÜ
pH PBK PDF PEP PPS PS PVC PVO
RMK RNA
SDI SDV SLV SKT SKP SMS SOS SRÜ SS STAS
TEPCO TM TMA TNT TPI TTÜ TV
UBS UCT UEFA UK UNHCR UPI URF URL US USA UV
ÕTÜ
ÜKT ÜRO
WC
);
my %akro_eesti = (); foreach (@akro_eesti) { $akro_eesti{$_} = 1; }

# inglispäraselt tähthaaval hääldatavad akronüümid
# lubatud on ka väiketäheline käändelõpp kriipsuga või ilma (TVs, BBC-ile)
# samuti liitsõna esiotsana kriipsuga (DVD-plaadid)
my @akro_inglise = qw (
ABC
BBC
CBS CNN
DJ
FBI
IBM
JFK
NBC NYPD
PR
UPN
YMCA YWCA
);
my %akro_inglise = (); foreach (@akro_inglise) { $akro_inglise{$_} = 1; }

# eesti reeglitega terviksõnana hääldatavad akronüümid
# lubatud on ka väiketäheline käändelõpp kriipsuga või ilma (NATOs),
# samuti liitsõna esiotsana kriipsuga (DVD-plaadid)
my @akro_sona = qw (
ABBA AIDS ARK ASCII ASEAN
BAM
EFTA EKE EKI ELO EPA ERKI ERSO ETA EXPO
FIDE FIE FIFA
GAZ GITIS GULAG
IKEA
LED
NASA NASCAR NASDAQ NATO
OMON OPEC
PERH PIN PRIA
RAF RAM
SAS SARS SEB SEK SIM SPARC
TSIK TU
UEFA UFO UNESCO UNICEF
VAZ VIN VIP
WADA WiFi
);
my %akro_sona = (); foreach (@akro_sona) { $akro_sona{$_} = 1; }


my %lyhendid = (
'a' => 'aasta',
'a.' => 'aasta',
'a/ü' => 'ametiühing',
'A/ü' => 'ametiühing',
'cm' => 'sentimeetrit',
'cm.' => 'sentimeetrit',
'Dr' => 'doktor',
'Dr.' => 'doktor',
'dr' => 'doktor',
'eKr' => 'enne kristust',
'e.Kr' => 'enne kristust',
'e.Kr.' => 'enne kristust',
'g' => 'grammi',
'gr' => 'grammi',
'gr.' => 'grammi',
'Hr' => 'härra',
'Hr.' => 'härra',
'hr' => 'härra',
'jm' => 'ja muud',
'jm.' => 'ja muud',
'jms' => 'ja muu selline',
'jms.' => 'ja muu selline',
'jne' => 'ja nii edasi',
'jne.' => 'ja nii edasi',
'jpt' => 'ja paljud teised',
'jpt.' => 'ja paljud teised',
'jt' => 'ja teised',
'jt.' => 'ja teised',
'kg' => 'kilo',
'km' => 'kilomeetrit',
'km/h' => 'kilomeetrit tunnis',
'kr' => 'krooni',
'Krt' => 'korter',
'krt' => 'korter',
'lk' => 'lehekülg',
'lk.' => 'lehekülg',
'Lk' => 'lehekülg',
'Lk.' => 'lehekülg',
'm' => 'meetrit',
'milj' => 'miljonit',
'min' => 'minutit',
'mg' => 'milligrammi',
'ml' => 'milliliitrit',
'mm' => 'millimeetrit',
'm/s' => 'meetrit sekundis',
'Mr' => 'mister',
'Mr.' => 'mister',
'mr' => 'mister',
'Mrs' => 'missis',
'Mrs.' => 'missis',
'Ms' => 'miss',
'Ms.' => 'miss',
'nn' => 'niinimetatud',
'nn.' => 'niinimetatud',
'Nn' => 'niinimetatud',
'nr' => 'number',
'nr.' => 'number',
'nt' => 'näiteks',
'nt.' => 'näiteks',
'nö' => 'niiöelda',
'n-ö' => 'niiöelda',
'OY' => 'oo üü',
'pr' => 'proua',
'Pr.' => 'proua',
'Pr' => 'proua',
'prl' => 'preili',
'Prl' => 'preili',
'prof' => 'professor',
'prof.' => 'professor',
'Prof' => 'professor',
'Prof.' => 'professor',
'sek' => 'sekundit',
'sh' => 'sealhulgas',
's.h' => 'sealhulgas',
's.h.' => 'sealhulgas',
'sh.' => 'sealhulgas',
'Sm' => 'seltsimees',
'Sm.' => 'seltsimees',
'sm' => 'seltsimees',
'sm.' => 'seltsimees',
's.o' => 'see on',
's.o.' => 'see on',
'St' => 'saant',
'St.' => 'saant',
's.t' => 'see tähendab',
's.t.' => 'see tähendab',
'tel' => 'telefon',
'tel.' => 'telefon',
'ts/ha' => 'tsentnerit hektarilt',
'u' => 'umbes',
'u.' => 'umbes',
'vm' => 'või muu',
'vm.' => 'või muu',
'vms' => 'või muu selline',
'vms.' => 'või muu selline',
'vt' => 'vaata',
'vt.' => 'vaata',
);

sub lyhend {
    my $l = shift;
    return (
	$lyhendid{$l} or $akro_eesti{$l} or $akro_inglise{$l} or $akro_sona{$l}
    );
}

sub sonadeks {
    my $l = shift;
    my $algne = $l;

    return $lyhendid{$algne} if $lyhendid{$algne};

    # votan punkti(d) lopust, meie lyhendites seda ei esine
    $l = $` if $l =~ /\.+$/;

    return $lyhendid{$l} if $lyhendid{$l};
    return $l if $akro_sona{$l};
    return tahthaaval($l) if $akro_eesti{$l};
    return spelling($l) if $akro_inglise{$l};

    return $algne;
}

sub tahthaaval {
    my $s = shift;
print "String $s\n";
    my $haaldus = '';
    foreach my $c (split //, $s) {
print "Taht $c ";
	$haaldus .= ' ' if $haaldus;
	$haaldus .= $tahed_eesti{lc($c)};
print "Haaldus '$haaldus'\n";
    }
    return $haaldus;
}

sub spelling {
    my $s = shift;
    $s = lc ($s);
    my $haaldus = '';
    foreach (split (//, $s)) {
	$haaldus .= ' ' if $haaldus;
	$haaldus .= $tahed_inglise{$_};
    }
    return $haaldus;
}

# väljakutsumine kas ($tyvi, $lopp) või ($lopp)
# tagastab 0/1
#
sub on_kaandelopp {
    my $tyvi = shift;
    return 0 unless defined ($tyvi);
    my $lopp = shift;
    if (! defined $lopp) {
	$lopp = $tyvi;
	$tyvi = '';
    }

    # Üldiselt on see kutsuja asi, kas tühi lõpp on lõpp või ei ole mitte
    # Kui tagastada 1, on ehk mugavam teha tsükleid tüvi+lõpp, ja kui on lõpp,
    # siis otsi tüve sõnastikust. Tyvi+0 puhul tuleb ka sõnastikku vaadata.
    return 1 if $lopp eq '';

    # ülakoma ja kriipsu järel lubame ainult 0- või i-omastavat
    # GPS-iga, J-le, LA-d, WC-deks, ...
    # häälduse lõpus i: Versaille', Versaille'i
    # häälduse lõpus konsonant: Whipple'iga
    return 0 if $lopp eq '-';	# ainult paljast kriipsu ei taha
    return 1 if $lopp =~ /^\'(i)?$/o;
    return 1 if $lopp =~ /^(\'|\-)i(d|t|de|te|sid)?$/o;
    return 1 if $lopp =~ /^(\'|\-)i(de|te)?(sse|s|st|le|l|lt|ks|ni|na|ta|ga)$/o;
    # Oprah't, WC-s
    return 1 if $lopp =~ /^(\'|\-)(d|t|de|te|id|sid)?$/o;
    return 1 if $lopp =~ /^(\'|\-)(de|te)?(sse|s|st|le|l|lt|ks|ni|na|ta|ga)$/o;
    # ???
    # bowling'ut

    my $vokaal = '(i)';	# aga võiks ka (a|e|i|u)
    # u: Henrikut, Cedricule
    $vokaal = '(i|u)' if $tyvi =~ /r[ei][kc]$/;
    # e: Jonas, Mooses, Hippokrates, Sokrates
    # a:

    # kõige prostam omastav
    return 1 if $lopp =~ /^$vokaal$/o;
    return 0 if $lopp eq 'iid';	# aid, eid, uid oleks siiski järgmisega lubatud
    return 1 if $lopp =~ /^$vokaal?(d|t|de|te|id|sid)?$/o;
    return 1 if $lopp =~ /^$vokaal(de|te)?(sse|s|st|le|l|lt|ks|ni|na|ta|ga)$/o;

    # vaikimisi: see ei olnud käändelõpp
    return 0;
}

__END__

=head1 NAME

Lingua::ET::Numbers - Perl extension to convert arabic or roman numerals to text in Estonian.

=head1 SYNOPSIS

    use Lingua::ET::Numbers;
    print number_to_word ('12,002');
    print number_to_word (1001, 'gen', 'ord');
    print number_to_word ('iv', 'ord');

=head1 DESCRIPTION

The C<Lingua::ET::Numbers> module exports just one function which can
be tuned to output numerals, either cardinal or ordinal in all cases.
The output string is tuned to mimic the way a human would read the
number out loud.

=head2 EXPORT

C<number_to_word> by default.

You can further export C<rom_to_int> to convert a correctly* formed Roman
numeral to integer.

* Despite popular opinion, 1999 cannot be written as IM.

=head1 PARAMETERS

First parameter is a numeral. It is the callers responsibility to
format it to (+|-)?(\d+)([.,]\d+)?
'1,000,000', '1 000 000', '- 3' etc are returned unchanged.

Without any additional parameters, the returned string corresponds to
a cardinal in nominative case.

=item ordinals

ord | ordinal | järgarv
card | cardinal | põhiarv (unnecessary, this is the default)

=item cases

nimetav    | nim | nom (again, the default)
omastav    | om  | gen
osastav    | os  | part
sisseütlev | sse | illatiiv
seesütlev  | s   | inessiiv
seestütlev | st  | elatiiv
alaleütlev | le  | allatiiv
alalütlev  | l   | adessiiv
alaltütlev | lt  | ablatiiv
saav       | ks  | translatiiv
rajav      | ni  | terminatiiv
olev       | na  | essiiv
ilmaütlev  | ta  | abessiiv
kaasaütlev | ga  | komitatiiv

=head1 ALGORITHM

Ugly.

Numbers in Estonian are sometimes compound words (200 == kakssada),
sometimes not (2000 == kaks tuhat). All the constituents decline,
even in compound words (200 in genitive is kahesaja).

There can be more than one ortographically legal way to decline:
200 in partitive is one of four variants of kaht(e?)sada(t?).
I have chosen the longest spellings, since this module is most
likely to be used in preparing texts for voice synthesis and longer
forms sound more unambiguous.

This module works with numbers up to 10^36 or so which will be
more than enough. Anything longer than that is just spelt digit by
digit.

=head1 VAGUE SPOTS

Plural is not implemented. Perhaps it will, if I see any need for it.

Ordinals are turned off for millions and higher numbers.

The use of ÜKS follows a general rule "output it only in the beginning
of the numeral". Thus "ÜKS miljon tuhat kolm" and not "ÜKS miljon
ÜKS tuhat kolm".

Practical use differs from the general rule "Only the last component
in illative to ablative, all the rest in genitive". When the numeral
is short, the first part usually assumes the same case. Cf
"kahe tuhande kolmesaja+le" and "kahe+le tuhande+le kolmesaja+le"
and even more widespread "kahe tuhande+le" vs commonly met
"kahe+le tuhande+le". In the absence of clear rules, this is implemented
for thousands and higher but only when this ends the sequence.
Thus 'nelja+ks tuhande+ks', but 'nelja tuhande üheks'

=head1 BUGS

=head1 AUTHOR

rom_to_int function is taken from the Roman module by Ozawa Sakuro and
Alexandr Ciornii.

Indrek Hein <lt>indrek.hein eki.ee<gt>


=head1 SEE ALSO

    Lingua::EN::Numbers
    Lingua::TR::Numbers
    Roman

=head1 COPYRIGHT

    Copyright (c) 2012 Indrek Hein. All rights reserved.

    This library is free software.
    You can redistribute it and/or modify it under the same terms as Perl itself.

=cut

