#!/usr/bin/perl -w

use strict;
use File::Pid;
use DBI();
use utf8;

# kutsutakse cron-ist kindla x-minutilise intervalliga, mis peab
# olema sama nii croni kui selle skripti jaoks. lopetab
# 5 sek enne, kui cron teda uuesti kutsuda tahab.
#
# Kui muudad crontabis, pane sama number parameetriks
# vaikimisi tootab 2 minutit

$| = 1;

my $log_uus_fail = 'Leidsin uue XML andmefaili';
my $log_silt_puudu = '<Filename> silt puudub!';
my $log_fail_puudu = 'Andmed tulid, aga STL fail ise puudub!';

my $minutit = ( ($#ARGV >= 0) && ($ARGV[0] =~ /[1-9]\d*/) ? $& : 2);
my $algusaeg = time;

my $inputdir = '/var/www/programmid/ftp/sissetulev';
my $queuedir = '/var/www/programmid/ftp/valmistood';
my $outputdir = '/var/www/programmid/ftp/valjaminev';
my $bindir = '/var/www/programmid/bin';

my $log = '/var/www/programmid/log/supa.log';
if (! -e $log) {
    `touch $log`;
    `chown www-data:www-data $log`;
}
open (LOG, ">>$log");
binmode (LOG, ":utf8");

my $pidfile = File::Pid->new( {file => '/var/run/cronftp.pid'} );
if ( $pidfile->running() ) {
    print LOG "cron_ftp.pl ei alusta, sest eelmine alles jookseb\n";
    die "Eelmine alles jookseb";
}
$pidfile->write();

while (time - $algusaeg < $minutit*60 - 5) {

    my @failid = <$inputdir/*>;
    foreach my $fn (@failid) {
	next unless $fn =~ /\.xml$/i;
	my $xmldata = '';
	open (F, "<$fn");
	binmode (F, ":utf8");
	while (<F>) { chomp; $xmldata .= $_; }
	close (F);
	print LOG $log_uus_fail, " $fn\n" if $log_uus_fail;
	$log_uus_fail = '';

	while ($xmldata =~ /<Subtitle( [^>]*)?>(.*?)<\/Subtitle>/i) {
	    $xmldata = $';
	    my $xmlkeel = $1;
	    my $xmlsupa = $2;
	    $xmlsupa .= "<Language>$xmlkeel</Language>" if $xmlkeel;
	    $xmlsupa .= "<Sourcexml>$fn</Sourcexml>";

	    if ($xmlsupa =~ /<Filename[^>]*>(.*?)<\/Filename>/i) {
		my $xmlfn = $1;

		my $stlfn = $inputdir . '/' . $xmlfn . '.srt';
		if (-e $stlfn) {
		    vota_toosse ($xmlsupa, $stlfn);
		    next;
		}

		$stlfn = $inputdir . '/' . $xmlfn . '.stl';
		if (-e $stlfn) {
		    vota_toosse ($xmlsupa, $stlfn);
		    next;
		}

		# peab taluma ka .Stl .SRT jms faililaiendeid. Lubame ühtlasi tõstutundetut nime
		foreach my $dirfn (@failid) {
		    next unless $dirfn =~ /\/$xmlfn\.(stl|srt)$/i;
		    $stlfn = $inputdir . $&;
		    if (-e $stlfn) {
			vota_toosse ($xmlsupa, $stlfn);
			next;
		    }
		}

		print LOG $log_fail_puudu, " ($stlfn .stl/.srt)\n" if $log_fail_puudu;
		$log_fail_puudu = '';
	    }
	    else {
		print LOG $log_silt_puudu, "\n" if $log_silt_puudu;
		$log_silt_puudu = '';
	    }
	}
    }

    # Valmis tööde väljastamine. Omanikuks tuleb kasutaja 'vota'
    @failid = <$queuedir/*>;
    if ( scalar(@failid) ) {
	print LOG "valmistood -> valjaminev\n";
	`chown -f vota $queuedir/*`;
	`mv -f $queuedir/* $outputdir/`;
    }

    sleep (1);
}

$pidfile->remove();

# ----------------------------------------------------------

# ja enne loppu veel yks pisike varskendus
use LWP::Simple;
get 'http://localhost/index.php/subtiitrid';

# ----------------------------------------------------------


sub parsexml {
    my $x = shift;
    my $href = shift;
    $href->{'title'} = $1 if $x =~ /<Title>(.*?)<\/Title>/i;
    $href->{'language'} = lc($1) if $x =~ /<Language>(.*?)<\/Language>/i;
    $href->{'language'} = lc($1) if $x =~ /<Language>.*?\"(.+?)\".*?<\/Language>/i;

    $href->{'date'} = '2013-01-01';
    $href->{'date'} = $3.'-'.$2.'-'.$1 if $x =~ /<Date>(\d{1,2}).(\d\d).(\d\d\d\d)<\/Date>/i;

    $href->{'time'} = '01:01:01:01';
    $href->{'time'} = $1 if $x =~ /<Time>(.*?)<\/Time>/i;

    $href->{'filename'} = $1 if $x =~ /<Filename>(.*?)<\/Filename>/i;
    $href->{'editor'} = $1 if $x =~ /<Editor>(.*?)<\/Editor>/i;
    $href->{'inpoint'} = $1 if $x =~ /<InPoint>(.*?)<\/InPoint>/i;
    $href->{'sourcexml'} = $1 if $x =~ /<Sourcexml>(.*?)<\/Sourcexml>/i;
}

sub vota_toosse {
    my $x = shift;	# andmed xml-stringina
    my $fn = shift;	# sisendfaili täistee

    print LOG "Võtan ftp-st töösse $fn\n";
    print LOG "Selle andmed: $x\n";

    return unless $fn =~ /.+\/(.+)/;
    my $stlnimi = $1;
    my $joomlafail = '/var/www/supa/media/contentbuilder/upload/' . $1;

    my %andmed = ();
    parsexml ($x, \%andmed);

    my $dbh = DBI->connect ("DBI:mysql:<BAAS>:localhost", "<KASUTAJA>", "<PAROOL>", {'RaiseError' => 1});
    $dbh->{'mysql_enable_utf8'} = 1;
    $dbh->do ("SET NAMES 'utf8'");
    print LOG "Logisin baasi sisse\n";

    # aga äkki on töötlus pooleli??
    my $poolik_sql = "SELECT * FROM j_subtiitrid WHERE (nimi = '" . $andmed{'filename'} . "') AND (staatus_heli < 101)";
    my ($poolikud) = $dbh->selectrow_array($poolik_sql);
    if ($poolikud) {
	print LOG "On hetkel kinni ($poolik_sql)\n";
	sleep(5);
	return;
    }

# 1. Tõstame faili Joomlasse
    `mv $fn $joomlafail`;
    print LOG "Tõstsin Joomlasse\n";

# 2. Paneme faili omanikuks www-data
    `chown www-data:www-data $joomlafail`;

# 3. Kustutame - kui on - eelmise töö
    # peaks tagama, et me kogemata kogu /work-i ei kustuta
    die "Faili nimi puudu/vigane" unless $andmed{'filename'} =~ /\w/;
    my $too = '/var/www/programmid/work/' . $andmed{'filename'};
    `mkdir $too`;
    `chown www-data:www-data $too`;
    `rm -f $too/*`;
    print LOG "Tekitasin/puhastasin töökataloogi\n";

# 4. Teeme xml-i töökataloogi ja seame omanikuks www-data
    my $andmefail = $andmed{'sourcexml'};
    if (-e $andmefail) {
        `chown www-data:www-data $andmefail`;
        `mv $andmefail $too/`;
	print LOG "Tõstsin sinna ka xml-i\n";
    }
    symlink ($joomlafail, "$too/$stlnimi");
    print LOG "Ja lisasin symlingi $too/$fn -> $joomlafail\n";

# 5. Registreerime baasis, et töötlus saaks alata
    my $sth = $dbh->prepare (qq{INSERT INTO j_subtiitrid (
	user_id, created, created_by,
	pealkiri, eetris_kuupaev, eetris_kellaaeg, ajanihe, keel, stl
	) values (?,?,?,?,?,?,?,?,?)}) or die "cron_ftp prepare kukkus: " . $dbh->errstr;
    print LOG "Valmistasin inserdi\n";
    $sth->execute(
	931, time(), 'Indrek Hein',
	$andmed{'title'}, $andmed{'date'}, $andmed{'time'}, $andmed{'inpoint'}, $andmed{'language'},
	$joomlafail
    );
    print LOG "Registreerisin baasis ja nüüd lõpetan\n";

}

