#!/usr/bin/perl -w

# See siin on puhtalt speller - otsib välja kõik, mida ei oska analüüsida
#
# parameetrid:
# -d		silumine
# -s		speller (siin : alati sees)
# -1		esimene sobiv (kah alati sees)
# -tNNN		lubatud tööaeg kuni NNN sekundit, vaikimisi 60
#		-t0 == tööta kaua vaja
# tekstifaili nimi
#
# Katkestamise tööpõhimõte:
# Alustame alati kogu tekstist sonaloendi tegemisega, see on piisavalt kiire
# ja tagab ka teksti muutmise puhul tulemuse.
# Samasse kataloogi tehakse kaks loendit: .err vigadega ja .ok tulemuslikega.
# Igal käivitamisel loeme need sisse ja teeme siis uued.
# .err laiendiga fail ongi lõpuks vajalik. Failides read kujul sõne<tab>sagedus.

use utf8;
use Storable;
$| = 1;

###############

my $stat_algusaeg = time;
my $sekundeid = 60;
my $dbg = 0;		# väljasta móttekäik
my $err = 1;
my $esi = 1;

# kóik käsureal olevad asjad käiatakse läbi ja otsitakse boolean parameetreid
foreach my $a (0..$#ARGV) {
    $dbg = $dbg || ($ARGV[$a] eq '-d');
    $sekundeid = $1 if ($ARGV[$a] =~ /^\-t(\d+)$/);
}

# Initsialiseerimine

my $morfdatadir = $0;
$morfdatadir = $& . 'morf_data/' if $morfdatadir =~ /.*\//;

my $fail_formtab = 	$morfdatadir . 		'formtab.pmf';
my $fail_formtablisa =  $morfdatadir . 		'dertab.pmf';
my $fail_tyvebaas =  	$morfdatadir . 		'tyvebaas.pmf';
my $fail_liitsonajupid = $morfdatadir . 	'liitsonajupid.pmf';
my $fail_erandid =  	$morfdatadir . 		'vormierandid1.pmf';
my $fail_vormierandid = $morfdatadir . 		'vormierandid2.pmf';
my $fail_lisad =  	$morfdatadir . 		'lisad.pmf';

my @formtab = ();
my %tyvebaas = ();
my %lsjupid = ();
my %erandid = ();
my %muutumatud = ();	# selle taidame TB lugemise kaigus
my %lisad = ();

my ($fn, $dfn);

# true, kui andmefaili dump on kasutuskolblik
sub varske {
    my $fn = shift;
    my $dfn = shift;
    $dfn = $fn . '.dmp' unless $dfn;
    return ( (-e $fn) and (-e $dfn) and ((stat($fn))[9] < (stat($dfn))[9]) );
}

$fn = $fail_liitsonajupid;
$dfn = $fn . '.dmp';
if ( varske ($fn) ) {
    %lsjupid = %{ retrieve ($dfn) };
}
else {
    loesisse_liitsonajupid($fn);
    store \%lsjupid, $dfn;
    print "salvestasin liitsonajupid uuesti\n" if $dbg;
}

$fn = $fail_formtab;
$dfn = $fn . '.dmp';
if ( varske($fail_formtab) and varske($fail_formtablisa, $dfn) ) {
    @formtab = @{ retrieve ($dfn) };
}
else {
    loesisse_formtab($fail_formtab);
    loesisse_formtab($fail_formtablisa);
    store \@formtab, $dfn;
}

$fn = $fail_erandid;
$dfn = $fn . '.dmp';
if ( varske($fail_erandid) and varske($fail_vormierandid, $dfn) ) {
    %erandid = %{ retrieve ($dfn) };
}
else {
    loesisse_erandid($fn);
    loesisse_vormierandid($fail_vormierandid);
    store \%erandid, $dfn;
    print "salvestasin erandid uuesti\n" if $dbg;
}

$fn = $fail_tyvebaas;
$dfn = $fn . '.dmp';
if ( varske($fn) ) {
    %tyvebaas = %{ retrieve ($dfn) };
    %muutumatud = %{ retrieve ($morfdatadir . 'muutumatud.pfm.dmp') };
}
else {
    loesisse_tyvebaas($fn);
    store \%tyvebaas, $dfn;
    store \%muutumatud, $morfdatadir . 'muutumatud.pfm.dmp';
    print "salvestasin tyvebaasi ja muutumatud uuesti\n" if $dbg;
}

$fn = $fail_lisad;
open (F, "<$fn") or die "puudub $fn!";
binmode (F, ":utf8");
while (<F>) { chomp; $lisad{ $_ } = 1; }


my %leitud = ();	# peame meeles lahenduvaid analyyse
my %leidmata = ();	# peame meeles mittelahenduvaid analyyse
my %varemleitud = ();	# eelmise ringi lahenduvad analyysid
my %varemleidmata = ();	# eelmise ringi mittelahenduvad analyysid




foreach my $a (0..$#ARGV) { 
    analyysi_fail ($ARGV[$a]) unless $ARGV[$a] =~ /^\-/;
}




###############

sub asendataheklassid {
    my $tingimus = shift;
    if ($tingimus =~ /%[A-Z]/) {
	return asendataheklassid($`) . $& . asendataheklassid($');
    }

    $tingimus =~ s/W/[aeiouõäöüy]/go;
    $tingimus =~ s/E/[aeu]/go;
    $tingimus =~ s/I/[aei]/go;
    $tingimus =~ s/U/[iuü]/go;
    $tingimus =~ s/Y/[aiõ]/go;
    $tingimus =~ s/X/[eouäöü]/go;
    $tingimus =~ s/A/[aeiou]/go;
    $tingimus =~ s/O/[õäöü]/go;
    $tingimus =~ s/C/[bcdfghjklmnpqrsšzžtvwx]/go;
    $tingimus =~ s/Q/[kptfš]/go;
    $tingimus =~ s/G/[gbd]/go;
    $tingimus =~ s/K/[kpt]/go;
    $tingimus =~ s/Z/[gbdlmnrvshjzž]/go;
    $tingimus =~ s/H/[lmnrvshjzž]/go;
    $tingimus =~ s/V/[lmnrvwž]/go;
    $tingimus =~ s/M/[lmnrv]/go;
    $tingimus =~ s/L/[lmnr]/go;
    $tingimus =~ s/N/[lnr]/go;
    $tingimus =~ s/R/[nr]/go;
    $tingimus =~ s/J/[lrvj]/go;
    $tingimus =~ s/T/[gbdkptlmnrvshj]/go;
    $tingimus =~ s/B/[gbdjfšzž]/go;
    $tingimus =~ s/D/[ntd]/go;
    $tingimus =~ s/F/[fš]/go;
    $tingimus =~ s/S/[sh]/go;
    $tingimus =~ s/Ü/[ie]/go;
    $tingimus =~ s/Ä/[ns]/go;
    $tingimus =~ s/Õ/[mjntv]/go;
    $tingimus =~ s/Ö/[dtslkng]/go;
    $tingimus =~ s/P/[gbdkptfh]/go;

    return $tingimus;
}

sub slurpin {
    my ($fn, $href) = @_;
    my $dfn = $fn . '.dmp';
    if ( (-e $fn) and (-e $dfn) and ((stat($fn))[9] < (stat($dfn))[9]) ) {
	$href = retrieve ($dfn);
    }
}

sub spitout {
    my ($fn, $href) = @_;
    my $dfn = $fn . '.dmp';
    store $href, $dfn;
}

sub loesisse_liitsonajupid {
    my $misnimeline = shift;
    open (F, "<$misnimeline") or die "puudub $misnimeline!";
    binmode (F, ":utf8");
    while (<F>) {
	chomp;
	$lsjupid{ $_ } = 1;
    }
}

sub loesisse_erandid {
    my $misnimeline = shift;
    open (F, "<$misnimeline") or die "puudub $misnimeline!";
    binmode (F, ":utf8");
    while (<F>) {
	chomp;
	next unless /^([^ ]+) .*?\(([^ ]+) .*?>(.+)/;
	$erandid{ $1 } = ":$2 $3:";
    }
}

sub loesisse_vormierandid {
    my $misnimeline = shift;
    open (F, "<$misnimeline") or die "puudub $misnimeline!";
    binmode (F, ":utf8");
    while (<F>) {
	chomp;
	my ($tk, $lemma, $vorm, $vormikood, $muu) = split (/,/, $_);
	next unless $vorm;
	# vormid, mida me EI tohiks moodustada jätame hetkel tähelepanuta.
	# vóiks muidugi kontrollida, et enese nimetavat käänet pole olemas vói
	# hea mitmuse osastav peab olema häid, mitte aga heasid, aga keelama ei hakka.
	next if $vorm eq '#';
	$vorm =~ s/[\'\[\]]//g;
	$erandid{ $vorm } .= ":$vormikood $lemma:";
    }
}

# LOE SISSE FORM.TAB
# jagame tyved eraldi kirjeteks ja kontrollime sealjuures korduste vältimiseks, kas selline juba oli

sub loesisse_formtab {
    my $misnimeline = shift;
    open (F, "<$misnimeline") or die "puudub $misnimeline!";
    binmode (F, ":utf8");
    my %vakantsid = ();
    my %jubaoli = ();
    my @fttemp = ();

    while (<F>) {
	chomp;
	next unless $_;		# tyhi rida
	next if /^[;#]/;	# kommentaar, alguses # voi ;
	next if /^\/\//;	# kommentaar, alguses //
	push @fttemp, $_;
	my ($formatiiv, $vormikood, $tingimus, $tyved) = split (/,/, $_, 4);
	# unless on vajalik ,-----0G mitmetähenduslikkuse tóttu
	$vakantsid{ $formatiiv . ',' . $vormikood } = $tyved unless $vakantsid{ $formatiiv . ',' . $vormikood };
    }

    foreach (@fttemp) {
	my ($formatiiv, $vormikood, $tingimus, $tyved) = split (/,/, $_, 4);
	$tingimus = asendataheklassid ($tingimus);
	if ($tyved =~ /^=/) { $tyved = $vakantsid{ $' }; }
# if ($formatiiv eq 'd') { print "$vormikood $tyved\n"; sleep (5); }
	foreach my $t (split (/,/, $tyved)) {
	    my $yksiktyvi = "$formatiiv,$vormikood,$tingimus,$t";
	    next if $jubaoli{$yksiktyvi};	# tuletusega osas on palju topelt -- 9~11 paralleelid, -(a|e|u) jms
	    $jubaoli{$yksiktyvi} = 1;
	    push @formtab, $yksiktyvi;
	    print "$yksiktyvi\n" if $dbg;
	}
    }
}

# LOE SISSE _TB.TXT

sub lisatuletustyved {
    my ($tyvi, $tyyp, $tyvekood, $lemma, $sliik) = @_;

    if ($tyyp eq '27') {
	if ($tyvekood eq 'a0') {
	    if ($tyvi =~ /tse$/) { $tyvekood = ':27d0tsus'; $tyvi = $`; goto UUSTT; }
	    if ($tyvi =~ /le$/) { $tyvekood = ':27d0lus'; $tyvi = $`; goto UUSTT; }
	    if ($tyvi =~ /ta$/) { $tyvekood = ':27d0tus'; $tyvi = $`; goto UUSTT; }
	    if ($tyvi =~ /da$/) { $tyvekood = ':27d0dus'; $tyvi = $`; goto UUSTT; }
	}
    }
    if ($tyyp eq '12') {
	if ($tyvekood eq 'b0') {
	    if ($tyvi =~ /lase$/) { $tyvekood = ':27d0lus'; $tyvi = $`; goto UUSTT; }
	}
    }
    if ($tyyp eq '10') {
	if ($tyvekood eq 'b0') {
	    if ($sliik eq 'A') {
		if ($tyvi =~ /stikuse$/) { $tyvekood = ':10d0stikkus'; $tyvi = $`; goto UUSTT; }
	    }
	}
    }
    if ($tyyp eq '02') {
	if ($tyvekood eq 'b0') {
	    if ($sliik eq 'A') {
		if ($tyvi =~ /bli$/) { $tyvekood = ':02d0blus'; $tyvi = $`; goto UUSTT; }
		if ($tyvi =~ /(a|e|u)$/) { $tyvekood = ':02d0us'; $tyvi = $`; goto UUSTT; }
	    }
	}
    }
    if ($tyyp eq '25') {
	if ($tyvekood eq 'bt') {
	    if ($sliik eq 'A') {
		if ($tyvi =~ /u$/) { $tyvekood = ':25d0us'; $tyvi = $`; goto UUSTT; }
	    }
	}
    }
    if ($tyyp eq '07') {
	if ($tyvekood eq 'bt') {
	    if ($sliik eq 'A') {
		if ($tyvi =~ /u$/) { $tyvekood = ':07d0us'; $tyvi = $`; goto UUSTT; }
	    }
	}
    }
    if ($tyyp eq '03') {
	if ($tyvekood eq 'bt') {
	    if ($sliik eq 'A') {
		if ($tyvi =~ /u$/) { $tyvekood = ':03d0us'; $tyvi = $`; goto UUSTT; }
	    }
	}
    }
    if ($tyyp eq '01') {
	if ($tyvekood eq 'a0') {
	    if ($sliik eq 'A') {
		if ($tyvi =~ /e$/) { $tyvekood = ':01d0us'; $tyvi = $`; goto UUSTT; }
	    }
	}
    }

    return;

UUSTT:
#    $tyvi = $`;
    $tyvekood .= "#$lemma:";
    $tyvebaas{$tyvi} .= $tyvekood;
    print "$tyvi == $tyvekood\n" if $dbg;
}

sub loesisse_tyvebaas {
	my $misnimeline = shift;
	return if keys (%tyvebaas);
	open (F, "<$misnimeline") or die "puudub $misnimeline!";
	binmode (F, ":utf8");
	%tyvebaas = ();
	while (<F>) {
		chomp;
		next unless $_;		# tyhi rida
		next if /^[;#]/;	# kommentaar, alguses # voi ;
		next if /^\/\//;	# kommentaar, alguses //

		# tyvebaasi rida
		my $tyved = '';		# (muutumatud on ilma eraldajata ja tyvedeta)
		next unless /^(.)([^ ]+) ([0-9][0-9])_([A-Y]+)/;
		my $lemma = $2;
		my $tyyp = $3;
		my $sliik = $4;
		$tyved = $';
		$tyved = $' if $tyved =~ /^\s+\|?\s+/;

		# valted ja rohud
		$lemma =~ s/[\'\`]//g; # `

		if ($tyved eq '') {
		    $muutumatud{$lemma} .= ":${tyyp}_${sliik}:";
		}

		foreach (split (/,/, $tyved)) {
			next unless /([a-y0]+): /;
			my $tyvi = $';
			my $tyvekood = $1;
			$tyvi =~ s/[\'\`]//g; # `

			$tyvebaas{$tyvi} .= ":$tyyp$tyvekood#$lemma#$sliik:";
			# print "$tyvi = $tyyp$tyvekood\n" if $tyvi =~ /aa/;
			#prots lisab %tyvebaasile kavatsema taha d0tsus: kava
			lisatuletustyved ($tyvi, $tyyp, $tyvekood, $lemma, $sliik);
		}
	}
}

# ANALYYSI

sub testitingimust {
    my ($algus, $tingimus) = @_;
    my $sonaliik = '';
    my $kustuta = 0;
    if ($tingimus =~ /%([A-Z])/) { $tingimus = $`; $sonaliik = $1; }
    if ($tingimus =~ /^\-/) { $tingimus = $'; $kustuta = 1; }
    if ($algus =~ /$tingimus$/) {
	print "algus: $algus, tingimus: $tingimus\n";
	$algus = $` if $kustuta;
	return $algus;
    } else { 
	return '';
    }
}

sub jaga_liitsonajuppideks {
    my $suurjupp = shift;
    return $suurjupp if $lsjupid{$suurjupp};
    return '' unless $suurjupp;

    my $i = 2;
    while ( $i <= length($suurjupp)-2 ) {
	my $sonaalgus = substr $suurjupp, 0, $i;
	my $sonalopp = substr $suurjupp, $i;
	$i++;
	next unless $lsjupid{$sonaalgus};
        $sonalopp = jaga_liitsonajuppideks ($sonalopp);
        next if $sonalopp eq '?';
	return "$sonaalgus+$sonalopp";
    }
    return '?';
}

sub puhastasona {
    my $mustsona = shift;
    $mustsona =~ s/[\.,!\?\":;]//g;
    $mustsona = $' if $mustsona =~ /^[\/\(]+/;
    $mustsona = $` if $mustsona =~ /[\/\)\-]+$/;
    return $mustsona;
}

sub analyysi_fail {
    my $fn = shift;
    if (-e $fn) {
	my $stat_sonesid = 0;
	my $stat_leidmata = 0;
	my $sn = 0;

	%leitud = ();	# peame meeles lahenduvaid analyyse
	%leidmata = ();	# peame meeles mittelahenduvaid analyyse

	# teeme kogu failist sõnaloendi: sone->sagedus
	open (F, "<$fn") or print "Ei saanud avada $fn\n";
	binmode (F, ":utf8");
	my %sonaloend = ();
	my %sonede_snumbrid = ();
	while (<F>) {
	    if (/^(\d+)\t/) {
		$sn = $1;
		$_ = $';
	    }
	    foreach my $sone (split(/\s+/, $_)) {
		$sone = puhastasona($sone);
		next unless $sone;
		$sonaloend{$sone}++;
		$sonede_snumbrid{$sone} .= ",$sn";
	    }
	}
	close (F);

	# Vaatame, kas varemleituid on
	if (-e "$fn.ok") {
	    open (F, "<$fn.ok");
	    binmode (F, ":utf8");
	    while (<F>) {
		chomp;
		my ($s, $n) = split (/\t/, $_);
		if ( $sonaloend{$s} ) {
		    $leitud{$s} = $n;
		    $sonaloend{$s} = 0;
		}
	    }
	    close (F);
	}

	# Vaatame, kas varemleidmata on
	if (-e "$fn.err") {
	    open (F, "<$fn.err");
	    binmode (F, ":utf8");
	    while (<F>) {
		chomp;
		my ($s, $n) = split (/\t/, $_);
		if ( $sonaloend{$s} ) {
		    $leidmata{$s} = $n;
		    $sonaloend{$s} = 0;
		}
	    }
	    close (F);
	}

	my $pooleli = 0;
	foreach my $sone (keys %sonaloend) {
	    if ( time > $stat_algusaeg + $sekundeid ) {
		$pooleli = 1;
		last;
	    }

	    my $sagedus = $sonaloend{$sone};
print "$sone\t$sagedus\n" if $dbg;
	    next unless $sagedus;


	    if ( analyysi ($sone) ) {
		$leitud{$sone} = $sagedus;
	    } else {
		$leidmata{$sone} = $sagedus;
	    }
	}

	open (F, ">$fn.err");
	binmode (F, ":utf8");
	foreach (keys %leidmata) {
	    print F "$_\t$leidmata{$_}\n";
	}
	close F;

	open (F, ">$fn.ok");
	binmode (F, ":utf8");
	foreach (keys %leitud) {
	    print F "$_\t$leitud{$_}\n";
	}
	close F;

	# Kui oleme omadega valmis, teeme uue faili arvatavate võõrnimedega
#	return if $pooleli;

	open (F, ">$fn.err.loend");
	open (FSN, ">$fn.err.sn");
	binmode (F, ":utf8");
	binmode (FSN, ":utf8");
	foreach (keys %leidmata) {
	    next if /^[a-zõäöüšž0-9\-]+$/;
	    print F "$_\t$leidmata{$_}\n";
	    print FSN "$_\t$sonede_snumbrid{$_}\n";
	}
	close F;
	close FSN;
    }
    else { die "Ei leia faili $fn!\n"; }
}

sub analyysi {
    my $miski = shift;
    return unless $miski;
    foreach my $algne_s (split(/\s+/, $miski)) {
	next unless $algne_s;
	$algne_s = puhastasona ($algne_s);
	$s = lc($algne_s);
	next unless $s;
	return 1 if $lisad{$s};

	$tulemusi = 0;
	analyysi_liitsona ($s);

	return $tulemusi;
    }
}

sub analyysi_liitsona {
    my $sona = shift;
GI:
    my $i = 0;
    while ( $i <= length($sona)-2 ) {
	my $sonaalgus = substr $sona, 0, $i;
	my $sonalopp = substr $sona, $i;
	if ($i++) {
	    print "liitsonaalus: $sonaalgus -> " if $dbg;
	    $sonaalgus = jaga_liitsonajuppideks ($sonaalgus);
	    print "$sonaalgus\n" if $dbg;
	    next if $sonaalgus eq '?';
	}
	analyysi_lihtsona ($sonalopp, $sonaalgus);
	return 1 if ($tulemusi and $esi);
    }

    # ki/gi kontroll
    if (
	($sona =~ /.*[bcdfghjkpsšzžtx](?=ki$)/) or
	($sona =~ /.*[aejlmnoruvwõäöü](?=gi$)/)
    ) { $sona = $&; goto GI; }

    return $tulemusi;
}

sub analyysi_lihtsona {
    my $sona = shift;
    my $liitsonaalgus = shift;
    $liitsonaalgus .= '+' if $liitsonaalgus;
    print "Lihtsona: $liitsonaalgus $sona\n" if $dbg;

    my $tyvebaasityvekoodid = '';

    if ( $muutumatud{$sona} ) {
	$tyvebaasityvekoodid = $muutumatud{$sona};
	while ($tyvebaasityvekoodid =~ /:([^:]+):/) {
	    $tyvebaasityvekoodid = $` . ':' . $';
	    my $tbinf = $1;
	    print "-------- $liitsonaalgus$sona $tbinf\n" unless $err;
	    $tulemusi++;
	    return if $esi;
	}
    }

    if ( $erandid{$sona} ) {
	$tyvebaasityvekoodid = $erandid{$sona};
	while ($tyvebaasityvekoodid =~ /:([^:]+):/) {
	    $tyvebaasityvekoodid = $` . ':' . $';
	    my $tbinf = $1;
	    print "$tbinf\n" unless $err;
	    $tulemusi++;
	    return if $esi;
	}
    }

# parem siiski lühemalt lópult pikema poole
#    my $i = 2;
#    while ( $i <= length($sona) ) {
    my $i = length($sona);
    while ( $i >= 2 ) {
	my $sonaalgus = substr $sona, 0, $i;
	my $sonalopp = substr $sona, $i;
	$i--;

	print "\nüritan jagada $sonaalgus + $sonalopp\n" if $dbg;
	my $tyvebaasityvekoodid = $tyvebaas{$sonaalgus};
	# jah, vahemalt yks selline tyvi on tyvebaasis olemas
	next unless $tyvebaasityvekoodid;
	print "\t... $sonaalgus on tüvebaasis: $tyvebaasityvekoodid\n" if $dbg;

	#jagame mugavamalt massiiviks
        my @tbkoodid = ();
        while ($tyvebaasityvekoodid =~ /:([^:]+):/) {
	    $tyvebaasityvekoodid = $` . ':' . $';
	    push @tbkoodid, $1;
	}

	# NB eeldab juba, et formtabis on tüvekoodid eraldi ridadeks jaotatud
	foreach (@formtab) {
	    my ($formatiiv, $vormikood, $tingimus, $fttk) = split (/,/, $_, 4);
	    next unless $formatiiv eq $sonalopp;
#	    print "\t... $sonalopp on formatiiv: $_\n" if $dbg;

	    foreach my $tbinf (@tbkoodid) {
#		print "\t\t$tbinf\n" if $dbg;
		my ($tbtk, $lemma, $sliik) = split (/#/, $tbinf);
#		print "\t\tkas tüvebaasi $tbtk on sama mis formtabi $fttk?\n" if $dbg;
		next unless $tbtk eq $fttk;
		# tk on yhine nii tyvel kui lopul
		print "$vormikood $liitsonaalgus$lemma + $formatiiv\n" unless $err;
		$tulemusi++;
		return if $esi;
	    }
	}
    }
}
