#!/usr/bin/perl -w

use strict;
use File::Pid;
use File::Copy;
use DBI();

# kutsutakse cron-ist kindla x-minutilise intervalliga, mis peab
# olema sama nii croni kui selle skripti jaoks. supa_cron lopetab
# 5 sek enne, kui cron teda uuesti kutsuda tahab.
#
# Kui muudad crontabis, pane sama number supa_cron parameetriks
# vaikimisi tootab 2 minutit

$| = 1;

my $minutit = ( ($#ARGV >= 0) && ($ARGV[0] =~ /[1-9]\d*/) ? $& : 2);

our $dbh = DBI->connect ("DBI:mysql:<BAAS>:localhost", "<KASUTAJA>", "<PAROOL>", {'RaiseError' => 1});
$dbh->{'mysql_enable_utf8'} = 1;
$dbh->do ("SET NAMES 'utf8'");

my ($q, $sth);

# ----------------------------------------------------------------
# globaalset sorti konstandid
# ----------------------------------------------------------------

# tildega ei tule mkdir toime
# võiks ka $0 alusel struktuuri ehitada
our $basedir = '/var/www/programmid';
our $bindir = $basedir . '/bin';

our $inputdir = $basedir . '/input';	mkdir ($inputdir) unless (-d $inputdir);
$inputdir = '/www/test/media/contentbuilder/upload';

our $workdir = $basedir . '/work';	mkdir ($workdir) unless -d $workdir;
our $logdir = $basedir . '/log';		mkdir ($logdir) unless -d $logdir;
our $logfile = $logdir . '/supa.log';

# heli synteesil: kui mitu festivali saab korraga käima lasta
# võiks olla tuumasid-1

my $protsessoreid = 4;

# ----------------------------------------------------------------
# programm
# ----------------------------------------------------------------

my $spellerisisend = '';	# stl2srt teeb, morf kasutab

if (-e $logfile) {
    open (LOG, ">>$logfile");
} else {
    open (LOG, ">$logfile");
}
binmode (LOG, ":utf8");

my $pidfile = File::Pid->new( {file => '/var/run/supa.pid'} );
if ( $pidfile->running() ) {
    logi ("supa_cron.pl ei alusta, sest eelmine alles jookseb");
    die "Eelmine alles jookseb";
}

$pidfile->write();
my $algusaeg = time;


# logi (teade)
# pane_staatus_init|morf|heli (toonimi, 1|0)
# ava_uued_tood ();
# sule_valmis_tood ();

require "$bindir/supa_protseduurid.pl";


logi ('');


while (time - $algusaeg < $minutit*60 - 5) {

    # esimeses jarjekorras votame uued asjad vastu
    # mine tea, need voivad hirmus pakilised olla, ja saab ka teksti kallale minna
    ava_uued_tood ();

    # teises jarjekorras puhastame tekste, leiame võõrnimed jms
    # analüüs käib ka suhtkoht kiiresti
    if (my $toonimi = leia_koige_pakilisem_morf()) {
	logi ("Analyysi vajab esmalt $toonimi, alustan");
	# 1 - töös
	pane_staatus_morf ($toonimi, 1);
	sleep(2);
	puhasta_tekst ($toonimi);
	spelli ($toonimi, $minutit*60-(time-$algusaeg)-5 ); # max lubatud aeg - juba tootatud aeg - 5 sek
	next;
    }

    # kolmandas jarjekorras lopetame valmistood
    sule_valmis_tood();

    # siis yritame uusi/muutunud asju synteesida
    if (my $toonimi = leia_koige_pakilisem_heli()) {
	pane_synteesilukk ($toonimi, 1);
	logi ("MAIN Synteesi vajab esmalt $toonimi, alustan");
	synteesi ($toonimi, $minutit*60-(time-$algusaeg)-5); # max lubatud aeg - juba tootatud aeg - 5 sek
	# suht ebaoluline on wav->mp3 teisendused, et IE audio tage mängiks
	wav2mp3 ($toonimi, $minutit*60-(time-$algusaeg)-5);
	pane_synteesilukk ($toonimi, 0);
	sleep(2);
	next;
    }

    mp3lisaks ($minutit*60-(time-$algusaeg)-5);

    sleep (1);
}

$pidfile->remove();
logi ("valmis");
close (LOG);
exit;

# ----------------------------------------------------------------

sub puhasta_tekst {
    my $toonimi = shift;
    logi ("LYHV Puhastan $toonimi");
    my $cmd = "$bindir/srt_puhastaja.pl " . $workdir . '/' . $toonimi . '/' . $toonimi . '.srt';
    logi ("SPEL Kaivitan $cmd");
    system ($cmd);
}

sub spelli {
    my $toonimi = shift;
    my $maxtooaeg = shift;
    my $cmd = "$bindir/morf.pl " . $workdir . '/' . $toonimi . '/' . $toonimi . '.txt -t' . $maxtooaeg;
    logi ("SPEL Kaivitan $cmd");

    system ($cmd);

    if ($? == -1) {
	logi ("SPEL Morf suri: " . $!);
    }
    elsif ($? == 1) {
	logi ("SPEL Morf kavatseb hiljem edasi teha");
    }
    else {
	logi ("SPEL Lasin teksti analysaatorist labi");
	pane_staatus_morf ($toonimi, 2);
    }

    uus_vigade_loend ($toonimi);
}

sub synteesi {
    my $toonimi = shift;
    my $maxtooaeg = shift;

    my $cmd = "$bindir/synt_scm.pl " . $workdir . '/' . $toonimi . '/' . $toonimi . '.srt -t ' . $maxtooaeg;
    logi ("SYNT Kaivitan $cmd");
    system ($cmd);

    if ($? == -1) {
	logi ("SPEL Syntees suri: " . $!);
	return;
    }
    logi ("SYNT Lasin teksti synteesist labi");

    sleep (2);

    my $protsent = 0;
    if (-e $workdir . '/' . $toonimi . '/syntees.ok') {
	open (F, "<" . $workdir . '/' . $toonimi . '/syntees.ok');
	$protsent = (<F>);
	close (F);
	chomp $protsent;

	logi ("Valmis $protsent protsenti");
	pane_staatus_heli ($toonimi, $protsent);
    } else {
	logi ("Vaatame kui palju valmis on");
	leia_pane_staatus_heli($toonimi);
    }
}

sub mp3lisaks {
    my $maxtooaeg = shift;
    $sth = $dbh->prepare ("SELECT nimi FROM j_subtiitrid WHERE staatus_heli = 100 ORDER BY eetris_kuupaev,eetris_kellaaeg DESC");
    $sth->execute();
    while (my @t = $sth->fetchrow_array()) {
	my $t = $t[0];
	pane_staatus_heli ($t, 101) if wav2mp3 ($t, $maxtooaeg);
    }
}
